@@ -204,7 +204,7 @@ class ReadElf(object):
 
             if isinstance(segment, InterpSegment):
                 self._emitline('      [Requesting program interpreter: %s]' %
-                    bytes2str(segment.get_interp_name()))
+                    segment.get_interp_name())
 
         # Sections to segments mapping
         #
@@ -221,7 +221,7 @@ class ReadElf(object):
             for section in self.elffile.iter_sections():
                 if (    not section.is_null() and
                         segment.section_in_segment(section)):
-                    self._emit('%s ' % bytes2str(section.name))
+                    self._emit('%s ' % section.name)
 
             self._emitline('')
 
@@ -248,7 +248,7 @@ class ReadElf(object):
         #
         for nsec, section in enumerate(self.elffile.iter_sections()):
             self._emit('  [%2u] %-17.17s %-15.15s ' % (
-                nsec, bytes2str(section.name), describe_sh_type(section['sh_type'])))
+                nsec, section.name, describe_sh_type(section['sh_type'])))
 
             if self.elffile.elfclass == 32:
                 self._emitline('%s %s %s %s %3s %2s %3s %2s' % (
@@ -292,11 +292,11 @@ class ReadElf(object):
 
             if section['sh_entsize'] == 0:
                 self._emitline("\nSymbol table '%s' has a sh_entsize of zero!" % (
-                    bytes2str(section.name)))
+                    section.name))
                 continue
 
             self._emitline("\nSymbol table '%s' contains %s entries:" % (
-                bytes2str(section.name), section.num_symbols()))
+                section.name, section.num_symbols()))
 
             if self.elffile.elfclass == 32:
                 self._emitline('   Num:    Value  Size Type    Bind   Vis      Ndx Name')
@@ -310,7 +310,7 @@ class ReadElf(object):
                 if (section['sh_type'] == 'SHT_DYNSYM' and
                         self._versioninfo['type'] == 'GNU'):
                     version = self._symbol_version(nsym)
-                    if (version['name'] != bytes2str(symbol.name) and
+                    if (version['name'] != symbol.name and
                         version['index'] not in ('VER_NDX_LOCAL',
                                                  'VER_NDX_GLOBAL')):
                         if version['filename']:
@@ -333,7 +333,7 @@ class ReadElf(object):
                     describe_symbol_bind(symbol['st_info']['bind']),
                     describe_symbol_visibility(symbol['st_other']['visibility']),
                     describe_symbol_shndx(symbol['st_shndx']),
-                    bytes2str(symbol.name),
+                    symbol.name,
                     version_info))
 
     def display_dynamic_tags(self):
@@ -353,13 +353,13 @@ class ReadElf(object):
             padding = 20 + (8 if self.elffile.elfclass == 32 else 0)
             for tag in section.iter_tags():
                 if tag.entry.d_tag == 'DT_NEEDED':
-                    parsed = 'Shared library: [%s]' % bytes2str(tag.needed)
+                    parsed = 'Shared library: [%s]' % tag.needed
                 elif tag.entry.d_tag == 'DT_RPATH':
-                    parsed = 'Library rpath: [%s]' % bytes2str(tag.rpath)
+                    parsed = 'Library rpath: [%s]' % tag.rpath
                 elif tag.entry.d_tag == 'DT_RUNPATH':
-                    parsed = 'Library runpath: [%s]' % bytes2str(tag.runpath)
+                    parsed = 'Library runpath: [%s]' % tag.runpath
                 elif tag.entry.d_tag == 'DT_SONAME':
-                    parsed = 'Library soname: [%s]' % bytes2str(tag.soname)
+                    parsed = 'Library soname: [%s]' % tag.soname
                 elif tag.entry.d_tag.endswith(('SZ', 'ENT')):
                     parsed = '%i (bytes)' % tag['d_val']
                 elif tag.entry.d_tag.endswith(('NUM', 'COUNT')):
@@ -410,7 +410,7 @@ class ReadElf(object):
 
             has_relocation_sections = True
             self._emitline("\nRelocation section '%s' at offset %s contains %s entries:" % (
-                bytes2str(section.name),
+                section.name,
                 self._format_hex(section['sh_offset']),
                 section.num_relocations()))
             if section.is_RELA():
@@ -448,7 +448,7 @@ class ReadElf(object):
                         symbol['st_value'],
                         fullhex=True, lead0x=False),
                     '  ' if self.elffile.elfclass == 32 else '',
-                    bytes2str(symbol_name)))
+                    symbol_name))
                 if section.is_RELA():
                     self._emit(' %s %x' % (
                         '+' if rel['r_addend'] >= 0 else '-',
@@ -520,14 +520,14 @@ class ReadElf(object):
                             self._format_hex(offset, fieldsize=6,
                                              alternate=True),
                             verdef['vd_version'], flags, verdef['vd_ndx'],
-                            verdef['vd_cnt'], bytes2str(name)))
+                            verdef['vd_cnt'], name))
 
                     verdaux_offset = (
                             offset + verdef['vd_aux'] + verdaux['vda_next'])
                     for idx, verdaux in enumerate(verdaux_iter, start=1):
                         self._emitline('  %s: Parent %i: %s' %
                             (self._format_hex(verdaux_offset, fieldsize=4),
-                                              idx, bytes2str(verdaux.name)))
+                                              idx, verdaux.name))
                         verdaux_offset += verdaux['vda_next']
 
                     offset += verdef['vd_next']
@@ -541,7 +541,7 @@ class ReadElf(object):
                     self._emitline('  %s: Version: %i  File: %s  Cnt: %i' % (
                             self._format_hex(offset, fieldsize=6,
                                              alternate=True),
-                            verneed['vn_version'], bytes2str(verneed.name),
+                            verneed['vn_version'], verneed.name,
                             verneed['vn_cnt']))
 
                     vernaux_offset = offset + verneed['vn_aux']
@@ -556,7 +556,7 @@ class ReadElf(object):
                         self._emitline(
                             '  %s:   Name: %s  Flags: %s  Version: %i' % (
                                 self._format_hex(vernaux_offset, fieldsize=4),
-                                bytes2str(vernaux.name), flags,
+                                vernaux.name, flags,
                                 vernaux['vna_other']))
 
                         vernaux_offset += vernaux['vna_next']
@@ -573,7 +573,7 @@ class ReadElf(object):
                 section_spec))
             return
 
-        self._emitline("\nHex dump of section '%s':" % bytes2str(section.name))
+        self._emitline("\nHex dump of section '%s':" % section.name)
         self._note_relocs_for_section(section)
         addr = section['sh_addr']
         data = section.data()
@@ -616,7 +616,7 @@ class ReadElf(object):
                 section_spec))
             return
 
-        self._emitline("\nString dump of section '%s':" % bytes2str(section.name))
+        self._emitline("\nString dump of section '%s':" % section.name)
 
         found = False
         data = section.data()
@@ -662,6 +662,8 @@ class ReadElf(object):
             self._dump_debug_frames()
         elif dump_what == 'frames-interp':
             self._dump_debug_frames_interp()
+        elif dump_what == 'aranges':
+            self._dump_debug_aranges()
         else:
             self._emitline('debug dump not yet supported for "%s"' % dump_what)
 
@@ -717,7 +719,7 @@ class ReadElf(object):
             num_entries = version_section.num_symbols()
 
         self._emitline("\n%s section '%s' contains %s entries:" %
-            (name, bytes2str(version_section.name), num_entries))
+            (name, version_section.name, num_entries))
         self._emitline('%sAddr: %s  Offset: %s  Link: %i (%s)' % (
             ' ' * indent,
             self._format_hex(
@@ -725,8 +727,7 @@ class ReadElf(object):
             self._format_hex(
                 version_section['sh_offset'], fieldsize=6, lead0x=True),
             version_section['sh_link'],
-            bytes2str(
-                self.elffile.get_section(version_section['sh_link']).name)
+                self.elffile.get_section(version_section['sh_link']).name
             )
         )
 
@@ -785,12 +786,12 @@ class ReadElf(object):
                     index <= self._versioninfo['verdef'].num_versions()):
                 _, verdaux_iter = \
                         self._versioninfo['verdef'].get_version(index)
-                symbol_version['name'] = bytes2str(next(verdaux_iter).name)
+                symbol_version['name'] = next(verdaux_iter).name
             else:
                 verneed, vernaux = \
                         self._versioninfo['verneed'].get_version(index)
-                symbol_version['name'] = bytes2str(vernaux.name)
-                symbol_version['filename'] = bytes2str(verneed.name)
+                symbol_version['name'] = vernaux.name
+                symbol_version['filename'] = verneed.name
 
         symbol_version['index'] = index
         return symbol_version
@@ -807,7 +808,7 @@ class ReadElf(object):
                 return None
         except ValueError:
             # Not a number. Must be a name then
-            return self.elffile.get_section_by_name(str2bytes(spec))
+            return self.elffile.get_section_by_name(spec)
 
     def _note_relocs_for_section(self, section):
         """ If there are relocation sections pointing to the givne section,
@@ -836,7 +837,7 @@ class ReadElf(object):
     def _dump_debug_info(self):
         """ Dump the debugging info section.
         """
-        self._emitline('Contents of the .debug_info section:\n')
+        self._emitline('Contents of the %s section:\n' % self._dwarfinfo.debug_info_sec.name)
 
         # Offset of the .debug_info section in the stream
         section_offset = self._dwarfinfo.debug_info_sec.global_offset
@@ -874,7 +875,7 @@ class ReadElf(object):
                     # Unknown attribute values are passed-through as integers
                     if isinstance(name, int):
                         name = 'Unknown AT value: %x' % name
-                    self._emitline('    <%2x>   %-18s: %s' % (
+                    self._emitline('    <%x>   %-18s: %s' % (
                         attr.offset,
                         name,
                         describe_attr_value(
@@ -889,7 +890,7 @@ class ReadElf(object):
         """ Dump the (decoded) line programs from .debug_line
             The programs are dumped in the order of the CUs they belong to.
         """
-        self._emitline('Decoded dump of debug contents of section .debug_line:\n')
+        self._emitline('Decoded dump of debug contents of section %s:\n' % self._dwarfinfo.debug_line_sec.name)
 
         for cu in self._dwarfinfo.iter_CUs():
             lineprogram = self._dwarfinfo.line_program_for_CU(cu)
@@ -930,11 +931,19 @@ class ReadElf(object):
                     # readelf doesn't print the state after end_sequence
                     # instructions. I think it's a bug but to be compatible
                     # I don't print them too.
-                    self._emitline('%-35s  %11d  %18s' % (
-                        bytes2str(lineprogram['file_entry'][state.file - 1].name),
-                        state.line,
-                        '0' if state.address == 0 else
-                               self._format_hex(state.address)))
+                    if lineprogram['version'] < 4:
+                        self._emitline('%-35s  %11d  %18s' % (
+                            bytes2str(lineprogram['file_entry'][state.file - 1].name),
+                            state.line,
+                            '0' if state.address == 0 else
+                                self._format_hex(state.address)))
+                    else:
+                        self._emitline('%-35s  %11d  %18s[%d]' % (
+                            bytes2str(lineprogram['file_entry'][state.file - 1].name),
+                            state.line,
+                            '0' if state.address == 0 else
+                                self._format_hex(state.address),
+                            state.op_index))
                 if entry.command == DW_LNS_copy:
                     # Another readelf oddity...
                     self._emitline()
@@ -944,7 +953,7 @@ class ReadElf(object):
         """
         if not self._dwarfinfo.has_CFI():
             return
-        self._emitline('Contents of the .debug_frame section:')
+        self._emitline('Contents of the %s section:' % self._dwarfinfo.debug_frame_sec.name)
 
         for entry in self._dwarfinfo.CFI_entries():
             if isinstance(entry, CIE):
@@ -972,13 +981,51 @@ class ReadElf(object):
             self._emit(describe_CFI_instructions(entry))
         self._emitline()
 
+    def _dump_debug_aranges(self):
+        """ Dump the aranges table
+        """
+        aranges_table = self._dwarfinfo.get_aranges()
+        if aranges_table == None:
+            return
+        # seems redundent, but we need to get the unsorted set of entries to match system readelf
+        unordered_entries = aranges_table._get_entries()
+       
+        if len(unordered_entries) == 0:
+            self._emitline()
+            self._emitline("Section '.debug_aranges' has no debugging data.")
+            return
+            
+        self._emitline('Contents of the %s section:' % self._dwarfinfo.debug_aranges_sec.name)
+        self._emitline()
+        prev_offset = None
+        for entry in unordered_entries:
+            if prev_offset != entry.info_offset:
+                if entry != unordered_entries[0]:
+                    self._emitline('    %s %s' % (
+                        self._format_hex(0, fullhex=True, lead0x=False), 
+                        self._format_hex(0, fullhex=True, lead0x=False)))
+                self._emitline('  Length:                   %d' % (entry.unit_length))
+                self._emitline('  Version:                  %d' % (entry.version))
+                self._emitline('  Offset into .debug_info:  0x%x' % (entry.info_offset))
+                self._emitline('  Pointer Size:             %d' % (entry.address_size))
+                self._emitline('  Segment Size:             %d' % (entry.segment_size))
+                self._emitline()
+                self._emitline('    Address            Length')
+            self._emitline('    %s %s' % (
+                self._format_hex(entry.begin_addr, fullhex=True, lead0x=False), 
+                self._format_hex(entry.length, fullhex=True, lead0x=False)))
+            prev_offset = entry.info_offset
+        self._emitline('    %s %s' % (
+                self._format_hex(0, fullhex=True, lead0x=False), 
+                self._format_hex(0, fullhex=True, lead0x=False)))
+
     def _dump_debug_frames_interp(self):
         """ Dump the interpreted (decoded) frame information from .debug_frame
         """
         if not self._dwarfinfo.has_CFI():
             return
 
-        self._emitline('Contents of the .debug_frame section:')
+        self._emitline('Contents of the %s section:' % self._dwarfinfo.debug_frame_sec.name)
 
         for entry in self._dwarfinfo.CFI_entries():
             if isinstance(entry, CIE):
@@ -1050,7 +1097,7 @@ class ReadElf(object):
     def _emitline(self, s=''):
         """ Emit an object to output, followed by a newline
         """
-        self.output.write(str(s) + '\n')
+        self.output.write(str(s).rstrip() + '\n')
 
 
 SCRIPT_DESCRIPTION = 'Display information about the contents of ELF format files'
